/*
 * PrefReg - Preference Registry manager
 * Copyright (c) 2002 by Project Waffle / TSUJIKAWA Tohru <tsujikawa@tsg.ne.jp>
 *  
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *  
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

///////////////////////////////////////////////////////////////////////////////
// PrefReg.h


#pragma	once

#include	<vector>
#include	<string>
using namespace std;

#include	<windows.h>

#include	"wicommon.h"


enum PrefRegError {
	PREFREG_SUCCESS,					// ANZX
	PREFREG_WRITEFAIL,					// ݎs
	PREFREG_NOSUCHKEYVALUE,				// ̂悤value͂Ȃ
};


//
// PrefReg
//
//   WItalk2 WXgǂݏpB@\͒PȂ̂ɌBxB
//
class PrefReg {

	HKEY	keyroot;
	string	keyname;

	HKEY readopen();
	HKEY writeopen();
	void close(HKEY key);

public:

	PrefReg(const char* keyname, bool doCreate = false, HKEY keyroot = HKEY_CURRENT_USER);
	PrefReg(const string& keyname, bool doCreate = false, HKEY keyroot = HKEY_CURRENT_USER) { PrefReg(keyname.c_str(), doCreate, keyroot); }
	~PrefReg() {}

	PrefRegError exist();
	PrefRegError create();
	PrefRegError deleteAll();

	DWORD readDword(const char* valname, DWORD defvalue);
	DWORD readDword(const string& valname, DWORD defvalue) { return readDword(valname.c_str(), defvalue); }
	string readString(const char* valname, const char* defvalue);
	string readString(const string& valname, const char* defvalue) { return readString(valname.c_str(), defvalue); }
	string readString(const char* valname, const string& defvalue) { return readString(valname, defvalue.c_str()); }
	string readString(const string& valname, const string& defvalue) { return readString(valname.c_str(), defvalue.c_str()); }

	PrefRegError writeDword(const char* valname, DWORD value);
	PrefRegError writeDword(const string& valname, DWORD value) { return writeDword(valname.c_str(), value); }
	PrefRegError writeString(const char* valname, const char* defvalue);
	PrefRegError writeString(const string& valname, const char* defvalue) { return writeString(valname.c_str(), defvalue); }
	PrefRegError writeString(const char* valname, const string& defvalue) { return writeString(valname, defvalue.c_str()); }
	PrefRegError writeString(const string& valname, const string& defvalue) { return writeString(valname.c_str(), defvalue.c_str()); }

	PrefRegError existSubkey(const char* keyname);
	PrefRegError existSubkey(const string& keyname) { return existSubkey(keyname.c_str()); }
	PrefRegError deleteSubkey(const char* keyname);
	PrefRegError deleteSubkey(const string& keyname) { return deleteSubkey(keyname.c_str()); }

	PrefRegError existValue(const char* valname);
	PrefRegError existValue(const string& valname) { return existValue(valname.c_str()); }
	PrefRegError deleteValue(const char* valname);
	PrefRegError deleteValue(const string& valname) { return deleteValue(valname.c_str()); }

	vector<string> enumSubkey();
	vector<string> enumValue();

};
