/*
 * LIBGANA - library for GANA
 * Copyright (C) 1996-1997, TAGA Nayuta <nayuta@is.s.u-tokyo.ac.jp>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *  
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

///////////////////////////////////////////////////////////////////////////////
// lgList.h


#ifndef __lgList_h__
#define __lgList_h__


#include	<winsock2.h>
#include "lgWindows.h"

template<class T>
class lgListBase
{
  class node
  {
  public:
    node *Prev;
    node *Next;
    T* Value;
    node()
    {
      Prev = Next = NULL;
      Value = NULL;
    }
    node(node *prev, node *next, T* value)
    {
      Prev = prev;
      Next = next;
      Value = value;
    }
  };
  
protected:
  node *start_node;
  node *now_node;
  int now_index;
  int size;
  node *get_node(const int index) const
	{
		node *n;
		int i;
		BOOL d;
		  
		if (size <= index)
			return NULL; // out of range
		if (index < 0)
			return NULL; // out of range
		if (index < now_index){
			if (index-0 < now_index-index){
				i = 0, n = start_node, d = TRUE;
			}else{
				i = now_index, n = now_node, d = FALSE;
			}
		}else{
			if (index - now_index < size - index){
				i = now_index, n = now_node, d = TRUE;
			}else{
				i = size, n = start_node, d = FALSE;
			}
		}
		while (i != index){
			if (d){
				n = n->Next, i++;
			}else{
				n = n->Prev, i--;
			}
		}
		((lgListBase *)this)->now_index = index;
		return ((lgListBase *)this)->now_node = n;
	}
public:
  lgListBase()
  {
    start_node = now_node = NULL;
    now_index = -1;
    size = 0;
  }
  ~lgListBase()
  {
    while (SizeOf() > 0)
      Delete(0);
  }
  int Insert(const int index, T* value)
  {
	if (size)
	{
		node *n = get_node(index);
		if (!n)
			if (index == size)
				n = start_node;
			else
				return -1; // out of range
		
		n = new node(n->Prev, n, value);
		n->Prev->Next = n;
		n->Next->Prev = n;
		size++;
		now_node = n;
		if (index == 0)
			start_node = now_node;
		return now_index = index;
	}
	else
	{
		start_node = now_node = new node;
		start_node->Prev = start_node->Next = start_node;
		start_node->Value = value;
		size = 1;
		return now_index = 0;
	}
  }


  int Delete(const int index) // TCY 0 Ȃ, -1 Ԃ
	{
		node *n = get_node(index);
		if (!n)
			return -1; // out of range
		if (size == 1)
		{
			delete start_node;
			size = 0;
			start_node = now_node = NULL;
			now_index = -1;
			return index;
		}
		else
		{
			if (n == start_node)
				start_node = n->Next;
			n->Prev->Next = n->Next;
			n->Next->Prev = n->Prev;
			size--;
			now_node = n->Next;
			now_index %= size;
			delete n;
			return index;
		}
	}
  T *Value (const int index) const
    // TCY 0 Ȃ, NULL Ԃ, 
    // Xg NULL ͊i[ł̂ŐMpĂ͂Ȃ
    {
		node *n = get_node(index);
		if (n)
			return n->Value;
		else
			return NULL;
	}

  int SizeOf() const
  {
    return size;
  }
  operator int() const
  {
    return SizeOf();
  }
};


///////////////////////////////////////////////////////////////////////////////
// lgPointerList


template<class T, class T2> class lgPointerList : public lgListBase<T2>
// Ex: lgPointerList<char *>;//lgPointerList<char*, char>
//     lgValueList<int>;
// (*0)
//     TCY 0 Ȃ, (T)NULL Ԃ, 
//     Xg NULL ͊i[ł̂ŐMpĂ͂Ȃ
// (*1)
//     TCY 0 Ȃ, Shift()-1 Ԃ
{
protected:
  int shift;
  
public:
  lgPointerList()
  {
    shift = 0;
  }
  int Insert(const int index, T value)
  {
    return lgListBase<T2>::Insert(index - shift, value) + shift;
  }
  int Insert(T value)
  {
    return lgListBase<T2>::Insert(0, value) + shift;
  }
  int Add(T value)
  {
    return lgListBase<T2>::Insert(Length(), value) + shift;
  }
  int Delete(const int index) // (*1)
  {
    return lgListBase<T2>::Delete(index - shift) + shift;
  }
  int Delete() // (*1)
  {
    return lgListBase<T2>::Delete(0) + shift;
  }
  T Value(const int index) const // (*0)
  {
    return lgListBase<T2>::Value(index - shift);
  }
  T operator[](const int index) const // (*0)
  {
    return lgListBase<T2>::Value (index - shift);
  }
  int Length() const
  {
    return SizeOf();
  }
  int SetShift(const int _shift)
  {
    int s = _shift;
    shift = _shift;
    return s;
  }
  int Shift() const
  {
    return shift;
  }
};
#define lgValueList lgPointerList


///////////////////////////////////////////////////////////////////////////////
// lgList


template<class T> class lgList : public lgListBase<T>
// Note: class T must have:
//       T(const T &);
// (*2)  TCY 0 Ȃ, Shift()-1 Ԃ
// (*3)  TCY 0 Ȃ, Segmentation Fault
{
protected:
  int shift;
  
public:
  lgList()
  {
    shift = 0;
  }
  ~lgList()
  {
    Clear();
  }
  int Insert(const int index, const T &value)
  {
    return lgListBase<T>::Insert(index - shift, new T(value)) + shift;
  }
  int Insert(const T &value)
  {
    return Insert(shift, value);
  }
  int Add(const T &value)
  {
    return lgListBase<T>::Insert(Length(), new T(value)) + shift;
  }
  int Delete(const int index) // (*2)
  {
    delete lgListBase<T>::Value(index - shift);
    return lgListBase<T>::Delete(index - shift) + shift;
  }
  int Delete() // (*2)
  {
    return Delete(shift);
  }
  void Clear()
  {
    while (Length())
      Delete();
  }
  T &Value(const int index) const // (*3)
  {
    return *lgListBase<T>::Value (index - shift);
  }
  T &operator[](const int index) const // (*3)
  {
    return Value(index);
  }
  int Length() const
  {
    return SizeOf();
  }
  int SetShift(const int _shift)
  {
    int s = _shift;
    shift = _shift;
    return s;
  }
  int Shift() const
  {
    return shift;
  }
};


///////////////////////////////////////////////////////////////////////////////
// lgSortedList


template<class T> class lgSortedList : public lgList<T>
// Note: class T must have:
//       T(const T &);
//       operator<=(const T &) const;
{
public:
  int Insert(const T &value)
  {
    if (SizeOf())
    {
      int i = shift;
      if ((*this)[i + now_index] <= value) i += now_index;
      while (1)
      {
	if (i - shift == SizeOf())
	  return lgList<T>::Insert(i, value);
	if (value <= (*this)[i])
	  return lgList<T>::Insert(i, value);
	i++;
      }
    }
    else
      return lgList<T>::Insert(value);
  }
  lgSortedList<T> &operator<<(const T &value)
  {
    Insert(value);
    return *this;
  }
};


///////////////////////////////////////////////////////////////////////////////
// Stack


template<class T> class lgStack : public lgListBase<T>
// Note: class T must have:
//       T(const T &);
// (*4)  TCY 0 Ȃ, Segmentation Fault
{
public:
  int Push(const T &value)
  {
    return Insert(0, (void *)new T(value));
  }
  T &Pop() // (*4)
  {
    T *t = (T *)Value(0);
    Delete(0);
    return *t;
  }
  lgStack<T> &operator<<(const T &value)
  {
    Push(value);
    return *this;
  }
  lgStack<T> &operator>>(T &value) // (*4)
  {
    value = Pop();
    return *this;
  }
};


///////////////////////////////////////////////////////////////////////////////
// Queue


template<class T> class lgQueue : public lgListBase<T>
// Note: class T must have:
//       T(const T &);
// (*5)  TCY 0 Ȃ, Segmentation Fault
{
public:
  lgQueue<T> &InsertFront(const T &value)
  {
    lgListBase::Insert(0, (void *)new T(value));
    return *this;
  }
  lgQueue<T> &InsertRear (const T &value)
  {
    lgListBase::Insert(SizeOf(), (void *)new T(value));
    return *this;
  }
  lgQueue<T> &Insert(const T &value)
  {
    return InsertRear(value);
  }
  T &DeleteFront() // (*5)
  {
    T *t = (T *)lgListBase::Value(0);
    lgListBase::Delete(0);
    return *t;
  }
  T &DeleteRear () // (*5)
  {
    T *t = (T *)lgListBase::Value(SizeOf() - 1);
    lgListBase::Delete(SizeOf() - 1);
    return *t;
  }
  T &Delete() // (*5)
  {
    return DeleteFront();
  }
  lgQueue<T> &operator<<(const T &value)
  {
    return InsertRear(value);
  }
  lgQueue<T> &operator>>(T &value) // (*5)
    // : >> ̎gp@ <, > ̎gp@ƈႢ܂
  {
    value = DeleteFront();
    return *this;
  }
  lgQueue<T> &operator<(const T &value)
  {
    return InsertRear(value);
  }
  lgQueue<T> &operator>(T &value) // (*5)
  {
    value = DeleteRear();
    return *this;
  }
};

template<class T> inline lgQueue<T> &operator>(const T &value,
					       lgQueue<T> &queue)
{
  return queue.InsertFront(value);
}

template<class T> inline lgQueue<T> &operator<(T &value,
					       lgQueue<T> &queue) // (*5)
{
  value = queue.DeleteFront();
  return queue;
}


#endif // __lgList_h__
