/*
 * LIBGANA - library for GANA
 * Copyright (C) 1996-1997, TAGA Nayuta <nayuta@is.s.u-tokyo.ac.jp>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *  
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

///////////////////////////////////////////////////////////////////////////////
// lgRegistry.cxx


#include "lgRegistry.h"
#include "lgFiles.h"

extern	char	iniFileName[BUFSIZE];

// waffle 0.1.4: INI t@CΉ
char	iniFileName[BUFSIZE] = ".\\waffle.ini";

  // Waffle 0.1.5:
  // DWORD ǂ INI t@Cɏ
BOOL lgRegistry::CopyDwordINI(const char* const name)
  {
	  int		ax;
	  char		tmp[256];

	  if (Read(root, path, name, ax, 4624)){
		  sprintf(tmp, "%d", ax);
		  return WritePrivateProfileString(path, name, tmp, iniFileName);
	  } else {
		  return FALSE;
	  }
  }
  // String ǂ INI t@Cɏ
BOOL lgRegistry::CopyStringINI(const char* const name)
  {
	  char		tmp[256];

	  if (Read(root, path, name, tmp, sizeof(tmp))){
		  return WritePrivateProfileString(path, name, tmp, iniFileName);
	  } else {
		  return FALSE;
	  }
  }

// Waffle 0.2.0f: String  INI ǂŃWXgɓ]ʂ
BOOL lgRegistry::CopyStringReg(HKEY root, const char* const path, const char* const name)
{
	char	tmp[256];

	GetPrivateProfileString(path, name, "4624x", tmp, sizeof(tmp), iniFileName);
	if (strcmp(tmp, "4624x") == 0) return false;
	Write(root, path, name, tmp);
	return true;
}

// 폜
BOOL lgRegistry::Delete(const HKEY root,
				 const char * const path,
				 const char * const name)
{
  if (path)
  {
    HKEY hkey;
    if (ERROR_SUCCESS == RegOpenKeyEx(root, path, 0, KEY_SET_VALUE, &hkey))
    {

	  // Waffle 0.1.3: Warning oȂ悤
#if 0
	  if (ERROR_SUCCESS == RegDeleteValue(hkey, name));
#endif
	  RegDeleteValue(hkey, name);

      {
	RegCloseKey(hkey);
	return TRUE;
      }
      RegCloseKey(hkey);
    }
  }
  return FALSE;
}


// DWORD ǂ
BOOL lgRegistry::Read(const HKEY root,
				const char * const path,
				const char * const name,
				int &buf, int default_value)
{
  if (path)
  {
    HKEY hkey;
    if (ERROR_SUCCESS == RegOpenKeyEx(root, path, 0, KEY_READ, &hkey))
    {
      DWORD type = REG_DWORD;
      DWORD size = sizeof(buf);
      if (ERROR_SUCCESS == RegQueryValueEx(hkey, name, NULL, &type,
					   (BYTE*)&buf, &size))
      {
	RegCloseKey(hkey);
	return TRUE;
      }
      RegCloseKey(hkey);
    }
  }

  // waffle 0.1.4: INI t@CΉ
  char	valueBuf[128];
  GetPrivateProfileString(path, name, "4624x", valueBuf, sizeof(valueBuf), iniFileName);
  if (strcmp(valueBuf, "4624x") != 0){
	  buf = atoi(valueBuf);
	  Write(root, path, name, buf);
	  return Read(root, path, name, buf, default_value);
  }

  buf = default_value;
  return FALSE;

}


// DWORD 
BOOL lgRegistry::Write(const HKEY root,
				 const char * const path,
				 const char * const name, int value)
{
  if (path)
  {
    HKEY hkey;
    DWORD disposition;

	// waffle 0.1.4: INI t@Cɏ
	//char	valueText[1024];
	//sprintf(valueText, "%d", value);
	//WritePrivateProfileString(path, name, valueText, iniFileName);

	if (ERROR_SUCCESS ==
	RegCreateKeyEx(root, path, 0, "REG_SZ", REG_OPTION_NON_VOLATILE,
		       KEY_ALL_ACCESS, NULL, &hkey, &disposition))
    {
      RegSetValueEx(hkey, name, NULL, REG_DWORD, (BYTE*)&value, sizeof(value));
      RegCloseKey(hkey);
      return TRUE;
    }
  }
  return FALSE;
}

int		CopiedIniVarLength = false;			// INI  WXg]ʂsꂽ
int		ReEntFlag = false;

// ǂ
BOOL lgRegistry::Read(const HKEY root,
				const char * const path,
				const char * const name,
				char *buf, const DWORD sizeof_buf,
				const char *default_str)
{

  if (path && buf && 0 < sizeof_buf)
  {

	// Waffle 0.2.0f: INI t@CƂ̊h
	//
	//   "ItalkServer0" "ColorPattern0" oɈȉ̏s:
	//
	//   1. WXg CopiedIniVarLength ȂA ItalkServer0 ݂Ȃ CopiedIniVarLength 𐶐AINI 烌WXgւ̕ʂs
	//   2. CopiedIniVarLength ݂ꍇAItalkServer?  INI t@Cǂݍ܂Ȃ
	//

	if (name){
		int		one = 1;

		Read(root, path, "CopiedIniVarLength", CopiedIniVarLength, false);

		if (!ReEntFlag && !CopiedIniVarLength){
			ReEntFlag = true;
			char	ItalkServer0[1024];
			Read(root, path, "ItalkServer0", ItalkServer0, sizeof(ItalkServer0), "4624x");
			if (strcmp(ItalkServer0, "4624x") == 0){
				/* INI  WXg͏ */
				for (int i = 0; ; i++){				// Italk server
					char	strtmp[256];
					sprintf(strtmp, "ItalkServer%d", i);
					if (!CopyStringReg(root, path, strtmp)) break;
				}
				for (int j = 0; ; j++){				// Fݒ
					char	strtmp[256];
					sprintf(strtmp, "ColorPattern%d", j);
					if (!CopyStringReg(root, path, strtmp)) break;
				}
				for (int k = 0; ; k++){				// killname
					char	strtmp[256];
					sprintf(strtmp, "KillName%d", k);
					if (!CopyStringReg(root, path, strtmp)) break;
				}
				for (int l = 0; ; l++){				// biff
					char	strtmp[256];
					sprintf(strtmp, "SoundPattern%d", l);
					if (!CopyStringReg(root, path, strtmp)) break;
				}
			}
			CopiedIniVarLength = true;
			Write(root, path, "CopiedIniVarLength", one);
		}
	}


	HKEY hkey;
    if (ERROR_SUCCESS == RegOpenKeyEx(root, path, 0, KEY_READ, &hkey)){
		DWORD type = REG_SZ;
		DWORD size = sizeof_buf;
		if (ERROR_SUCCESS == RegQueryValueEx(hkey, name, NULL, &type, (BYTE*)buf, &size)){
			if (size < sizeof_buf) buf[size] = '\0';
			RegCloseKey(hkey);
			return TRUE;
		}
		RegCloseKey(hkey);
    }

	// Waffle 0.2.0f: ɃWXgɓ͂̂ ItalkServer? WXgɂȂ  INI t@C̒lԂĂ͂ȂȂ
	if (name && (_strnicmp(name, "ItalkServer", 11) == 0 ||
		_strnicmp(name, "ColorPattern", 12) == 0 ||
		_strnicmp(name, "KillName", 8) == 0 ||
		_strnicmp(name, "SoundPattern", 12) == 0)){
		if (default_str) strcpy(buf, default_str);
		return false;
	}

	// waffle 0.1.4: INI t@CΉ
	if (name){
		GetPrivateProfileString(path, name, "4624x", buf, sizeof_buf, iniFileName);
		if (strcmp(buf, "4624x") != 0){
			//MessageBox(0, buf, name, MB_OKCANCEL | MB_ICONWARNING);
			Write(root, path, name, buf);
			return Read(root, path, name, buf, sizeof_buf, default_str);
		}
	}

  }

	if (default_str) strcpy(buf, default_str);
	return FALSE;
}


// 
BOOL lgRegistry::Write(const HKEY root,
				 const char * const path,
				 const char * const name,
				 const char * const value)
{
  if (path && value)
  {
    HKEY hkey;
    DWORD disposition;

	// waffle 0.1.4: INI t@Cɏ
	//WritePrivateProfileString(path, name, value, iniFileName);

	if (ERROR_SUCCESS ==
	RegCreateKeyEx(root, path, 0, "REG_SZ", REG_OPTION_NON_VOLATILE,
		       KEY_ALL_ACCESS, NULL, &hkey, &disposition))
    {
      RegSetValueEx(hkey, name, NULL, REG_SZ, (BYTE*)value, strlen(value) + 1);
      RegCloseKey(hkey);
      return TRUE;
    }
  }
  return FALSE;
}


// Chǂ
BOOL lgRegistry::Read(const HKEY root,
				const char * const path,
				const char * const name,
				wchar_t *buf, const DWORD sizeof_buf,
				const wchar_t *default_str)
{
  if (sizeof_buf <= 0)
    return FALSE;
  char *mbsbuf = (char *)_alloca(sizeof_buf * 2 + 1);
  if (Read(root, path, name, mbsbuf, sizeof_buf * 2 + 1, NULL))
  {
    if (0 <= mbstowcs(buf, mbsbuf, sizeof_buf))
      return TRUE;
  }
  if (default_str)
    wcscpy(buf, default_str);
  return FALSE;
}


// Ch
BOOL lgRegistry::Write(const HKEY root,
				 const char * const path,
				 const char * const name,
				 const wchar_t * const value)
{

	// waffle 0.1.4: INI t@Cɏ
	//WritePrivateProfileString(path, name, value, iniFileName);

  if (!value)
    return FALSE;
  int len = wcslen(value);
  char *mbsvalue = (char *)_alloca(len * 2 + 1);
  if (wcstombs(mbsvalue, value, len + 1) < 0)
    return FALSE;
  return Write(root, path, name, mbsvalue);
}


// oCiǂ
BOOL lgRegistry::Read(const HKEY root,
				const char * const path,
				const char * const name,
				BYTE *buf, DWORD &sizeof_buf,
				const BYTE *default_binary,
				DWORD sizeof_default_binary)
{

  if (path && buf && 0 < sizeof_buf)
  {
    HKEY hkey;
    if (ERROR_SUCCESS == RegOpenKeyEx(root, path, 0, KEY_READ, &hkey))
    {
      DWORD type = REG_BINARY;
      if (ERROR_SUCCESS == RegQueryValueEx(hkey, name, NULL, &type,
					   (BYTE*)buf, &sizeof_buf))
      {
	RegCloseKey(hkey);
	return TRUE;
      }
      RegCloseKey(hkey);
    }
  }
  if (default_binary)
    CopyMemory(buf, default_binary, sizeof_default_binary);
  return FALSE;
}


// oCi
BOOL lgRegistry::Write(const HKEY root,
				 const char * const path,
				 const char * const name,
				 const BYTE * const value,
				 DWORD sizeof_value)
{
  if (path && value)
  {
    HKEY hkey;
    DWORD disposition;
    if (ERROR_SUCCESS ==
	RegCreateKeyEx(root, path, 0, "REG_BINARY", REG_OPTION_NON_VOLATILE,
		       KEY_ALL_ACCESS, NULL, &hkey, &disposition))
    {
      RegSetValueEx(hkey, name, NULL, REG_BINARY, value, sizeof_value);
      RegCloseKey(hkey);
      return TRUE;
    }
  }
  return FALSE;
}


// LOGFONT ǂ
BOOL lgRegistry::Read(const HKEY root,
				const char * const path,
				const char * const name,
				LOGFONT &lf,
				const char * const
				default_string_logfont)
{
  char buf[BUFSIZE];
  if (!lgRegistry::Read(root, path, name, buf, sizeof(buf), NULL))
  {
    if (!default_string_logfont)
      return FALSE;
    strcpy(buf, default_string_logfont);
  }
  lf.lfHeight         =       atoi(_mbstok(buf,  ","));
  lf.lfWidth          =       atoi(_mbstok(NULL, ","));
  lf.lfEscapement     =       atoi(_mbstok(NULL, ","));
  lf.lfOrientation    =       atoi(_mbstok(NULL, ","));
  lf.lfWeight         =       atoi(_mbstok(NULL, ","));
  lf.lfItalic         = (BYTE)atoi(_mbstok(NULL, ","));
  lf.lfUnderline      = (BYTE)atoi(_mbstok(NULL, ","));
  lf.lfStrikeOut      = (BYTE)atoi(_mbstok(NULL, ","));
  lf.lfCharSet        = (BYTE)atoi(_mbstok(NULL, ","));
  lf.lfOutPrecision   = (BYTE)atoi(_mbstok(NULL, ","));
  lf.lfClipPrecision  = (BYTE)atoi(_mbstok(NULL, ","));
  lf.lfQuality        = (BYTE)atoi(_mbstok(NULL, ","));
  lf.lfPitchAndFamily = (BYTE)atoi(_mbstok(NULL, ","));
  strcpy(lf.lfFaceName, _mbstok(NULL, ","));
  return TRUE;
}


// LOGFONT ǂ
BOOL lgRegistry::Write(const HKEY root,
				 const char * const path,
				 const char * const name,
				 const LOGFONT &lf)
{
  char buf[BUFSIZE];
  wsprintf(buf, "%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%s",
	   lf.lfHeight, lf.lfWidth, lf.lfEscapement, lf.lfOrientation,
	   lf.lfWeight, lf.lfItalic, lf.lfUnderline, lf.lfStrikeOut,
	   lf.lfCharSet, lf.lfOutPrecision, lf.lfClipPrecision, lf.lfQuality,
	   lf.lfPitchAndFamily, lf.lfFaceName);
  return lgRegistry::Write(root, path, name, buf);
}


// RECT ǂ
BOOL lgRegistry::Read(const HKEY root,
				const char * const path,
				const char * const name,
				RECT &rc, const RECT &default_rc)
{
  char buf[BUFSIZE];
  if (!lgRegistry::Read(root, path, name, buf, sizeof(buf), NULL))
  {
	rc = default_rc;
    return FALSE;
  }

  rc.left   = atoi(_mbstok(buf,  ","));
  rc.top    = atoi(_mbstok(NULL, ","));
  rc.right  = atoi(_mbstok(NULL, ","));
  rc.bottom = atoi(_mbstok(NULL, ","));
  return TRUE;
}


// RECT
BOOL lgRegistry::Write(const HKEY root,
				 const char * const path,
				 const char * const name, const RECT &rc)
{
  char buf[BUFSIZE];
  wsprintf(buf, "%d,%d,%d,%d", rc.left, rc.top, rc.right, rc.bottom);
  return lgRegistry::Write(root, path, name, buf);
}


// ֘AtꂽWXgڂ𓾂
BOOL lgRegistry::GetAssociated(const char *type, const char *path,
					 char *buf, int sizeof_buf)
{
  char typebuf[BUFSIZE];
  strcpy(typebuf, type);
  if (lgRegistry::Read(HKEY_CLASSES_ROOT, typebuf, NULL, buf, sizeof_buf, NULL) && *buf != 0)	// Waffle 0.2.0f: *buf != 0 `FbNȂƉîȂւĂ܂
    if (lgRegistry::GetAssociated(buf, path, buf, sizeof_buf))
      return TRUE;
  lgFiles::ToPath(typebuf);
  strcat(typebuf, path);
  lgFiles::PackPath(typebuf);
  return lgRegistry::Read(HKEY_CLASSES_ROOT, typebuf, NULL, buf, sizeof_buf, NULL);
}


// ֘AtR}hC𓾂
BOOL lgRegistry::GetAssociatedCommandline(const char *type,
						   char *commandline,
						   int sizeof_commandline,
						   const char *action)
{
  char buf[BUFSIZE];
  strcpy(buf, "\\shell\\");
  strcat(buf, action);
  strcat(buf, "\\command");
  return lgRegistry::GetAssociated(type, buf, commandline, sizeof_commandline);
}


// ֘AtR}hC̉
char *lgRegistry::InterpretAssociatedCommandline
(const char *filename, const char *commandline, char *buf, int sizeof_buf)
{
  char *b = buf;
  sizeof_buf--;
  while (*commandline)
  {
    if (sizeof_buf <= 0)
      return NULL;
    if (commandline[0] == '%')
    {
      if (commandline[1] == '1' ||
          commandline[1] == 'L' ||
	  commandline[1] == 'l')
      {
	strcpy(b, filename);
	int len = strlen(filename);
	b += len;
	sizeof_buf -= len;
      }
      else if (commandline[1] == '%')
      {
	*(b++) = '%';
	sizeof_buf--;
      }
      commandline += 2;
    }
    else
    {
      *(b++) = *(commandline++);
      sizeof_buf--;
      if (_mbclen((BYTE *)(commandline - 1)) == 2)
      {
	if (!*commandline || sizeof_buf <= 0)
	  return NULL;
	*(b++) = *(commandline++);
	sizeof_buf--;
      }
    }
  }
  *b = '\0';
  return buf;
}


// ֘AtACR𓾂
HICON lgRegistry::GetAssociatedIcon(const char *type,
					      const char *filename)
{
  char buf[BUFSIZE];
  if (!lgRegistry::GetAssociatedCommandline(type, buf, sizeof(buf)))
    return NULL;
  if (lgFiles::CommandlineToCommand(buf, buf, sizeof(buf)))
    if (!_mbsiequal("rundll32.exe", buf))
    {
      HICON hicon = ExtractIcon(lgWindows::hInst(), buf, 0);
      if (hicon)
	return hicon;
    }
  
  if (lgRegistry::GetAssociated(type, "\\DefaultIcon", buf, sizeof(buf)))
  {
    char *i = _mbsrchr(buf, ',');
    int index = 0;
    if (i)
    {
      *i = '\0';
      index = atoi(i + 1);
    }
    HICON hicon = ExtractIcon(lgWindows::hInst(), buf, index);
    if (hicon)
      return hicon;
  }
  
  lgRegistry::GetAssociatedCommandline(type, buf, sizeof(buf));
  if (strnicmp("rundll32.exe", buf, sizeof("rundll32.exe")))
  {
    char *b = _mbschr(buf, ' ');
    if (b)
    {
      b++;
      char *i = _mbschr(b, ',');
      if (i)
	*i = '\0';
      return ExtractIcon(lgWindows::hInst(), b, 0);
    }
  }
  
  if (filename)
  {
    WORD index = 0;
    strcpy(buf, filename);
    return ExtractAssociatedIcon(lgWindows::hInst(), buf, &index);
  }
  return NULL;
}
