/*
 * LIBGANA - library for GANA
 * Copyright (C) 1996-1997, TAGA Nayuta <nayuta@is.s.u-tokyo.ac.jp>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *  
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

///////////////////////////////////////////////////////////////////////////////
// lgWindow.h


#pragma	once

#include	"lgWindows.h"

class lgWindow : public lgWindows
{
protected:

  const HWND hwnd;
  
public:
  lgWindow(HWND _hwnd)
    : hwnd(_hwnd) { }
  virtual ~lgWindow() { }
  
  HWND Hwnd(void) const { return hwnd; }
  operator HWND() const { return hwnd; }
  HWND Hwnd(int id) const { return GetDlgItem(Hwnd(), id); }
  static HWND Hwnd(HWND hwnd, int id) { return GetDlgItem(hwnd, id); }
  HWND HwndOwner(void) const { return GetWindow(Hwnd(), GW_OWNER); }
  
  // Window Message

  // EBhE쐬ꂽ
  int wmCreate(CREATESTRUCT *cs) { return 0; }
  
public:
  // ̑
  LRESULT SendMessageToItem(int id, UINT uMsg, WPARAM wParam = 0,
			    LPARAM lParam = 0)
  { return SendMessage(Hwnd(id), uMsg, wParam, lParam);}
  LRESULT SendMessage(UINT uMsg, WPARAM wParam = 0, LPARAM lParam = 0)
  { return SendMessage(Hwnd(), uMsg, wParam, lParam); }
  static LRESULT SendMessage(HWND hwnd, UINT uMsg,
			     WPARAM wParam = 0, LPARAM lParam = 0)
  { return ::SendMessage(hwnd, uMsg, wParam, lParam);}
  
  BOOL EnableWindow(BOOL is_enabled = TRUE)
  { return ::EnableWindow(Hwnd(), !!is_enabled);}
  BOOL EnableWindow(int id, BOOL is_enabled = TRUE)
  { return ::EnableWindow(Hwnd(id), !!is_enabled);}
  BOOL EnableWindow(HWND hwnd, BOOL is_enabled = TRUE)
  { return ::EnableWindow(hwnd, !!is_enabled);}
  
  BOOL IsWindowEnabled(void)
  { return (BOOL)::IsWindowEnabled(Hwnd()); }
  BOOL IsWindowEnabled(int id)
  { return (BOOL)::IsWindowEnabled(Hwnd(id)); }
  BOOL IsWindowEnabled(HWND hwnd)
  { return (BOOL)::IsWindowEnabled(hwnd); }

public:
  
  // Window ʒu܂ňړ
  static BOOL MoveWindowVisible(HWND hwnd);
  static RECT MoveWindowVisible(RECT rc);
  BOOL MoveWindowVisible(void)
  { return MoveWindowVisible(Hwnd()); }
  
  // MoveWindow (TCY͕ύXȂ)
  static BOOL MoveWindow(HWND hwnd, int x, int y, int w, int h,
			 BOOL is_repaint = TRUE)
  { return ::MoveWindow(hwnd, x, y, w, h, is_repaint); }
  BOOL MoveWindow(int x, int y, int w, int h, BOOL is_repaint = TRUE)
  { return MoveWindow(Hwnd(), x, y, w, h, is_repaint); }

  // MoveWindow  RECT o[W
  static BOOL MoveWindow(HWND hwnd, RECT *rc,
				   BOOL is_repaint = TRUE);
  BOOL MoveWindow(RECT *rc, BOOL is_repaint = TRUE)
  { return MoveWindow(Hwnd(), rc, is_repaint); }
  
  // MoveWindow (TCY͕ύXȂ)
  static BOOL MoveWindow(HWND hwnd, int x, int y,
				   BOOL is_repaint = TRUE);
  BOOL MoveWindow(int x, int y, BOOL is_repaint = TRUE)
  { return MoveWindow(Hwnd(), x, y, is_repaint); }
  
  // ResizeWindow (ʒu͈ړȂ)
  static BOOL ResizeWindow(HWND hwnd, int w, int h,
				     BOOL is_repaint = TRUE);
  BOOL ResizeWindow(int w, int h, BOOL is_repaint = TRUE)
  { return ResizeWindow(Hwnd(), w, h, is_repaint); }
  
  // eEBhE̒ł̃EBhȆ傫Ԃ
  static BOOL GetChildWindowRect(HWND hwnd, RECT *rc);
  BOOL GetChildWindowRect(RECT *rc)
  { return GetChildWindowRect(Hwnd(), rc); }
  
  // ĕ`
  static void Redraw(HWND hwnd)
  {
    InvalidateRect(hwnd, NULL, TRUE);
    UpdateWindow(hwnd);
  }
  void Redraw(void) { Redraw(Hwnd()); }
  
  // ^CgZbg
  static BOOL SetWindowText(HWND hwnd, int num);
  BOOL SetWindowText(int num) { return SetWindowText(Hwnd(), num); }
  static BOOL SetWindowText(HWND hwnd, const char *text)
  { return ::SetWindowTextA(hwnd, text); }
//static BOOL SetWindowText(HWND hwnd, const wchar_t *text)
//{ return ::SetWindowTextW(hwnd, text); }
  BOOL SetWindowText(const char *text) { return SetWindowText(Hwnd(), text); }
//BOOL SetWindowText(const wchar_t *text) {return SetWindowText(Hwnd(), text);}
  BOOL SetWindowText(int id, const char *text)
  { return SetWindowText(Hwnd(id), text); }
//BOOL SetWindowText(int id, const wchar_t *text)
//{ return SetWindowText(Hwnd(id), text); }

  // ^CgQbg
  static int GetWindowText(HWND hwnd, char *buf, int sizeof_buf)
  { return ::GetWindowTextA(hwnd, buf, sizeof_buf); }
//static int GetWindowText(HWND hwnd, wchar_t *buf, int sizeof_buf)
//{ return ::GetWindowTextW(hwnd, buf, sizeof_buf); }
  int GetWindowText(char *buf, int sizeof_buf)
  { return GetWindowText(Hwnd(), buf, sizeof_buf); }
//int GetWindowText(wchar_t *buf, int sizeof_buf)
//{ return GetWindowText(Hwnd(), buf, sizeof_buf); }
  int GetWindowText(int id, char *buf, int sizeof_buf)
  { return GetWindowText(Hwnd(id), buf, sizeof_buf); }
//int GetWindowText(int id, wchar_t *buf, int sizeof_buf)
//{ return GetWindowText(Hwnd(id), buf, sizeof_buf); }
};


// lgMainWindow class


/*
 * WinHelp g߂ɂ́A
 * 1. RXgN^ SetHelp() Ă
 * 2. EBhEvV[W switch  lgwmHelp, lgwmContextMenu,
 *    lgwmCommand Ă͑֐ŌĂ
 */

/*
 * toolbar g߂ɂ
 * 1. RXgN^ toolbar = lgcToolbar::newCreate() Ă
 * 2. fXgN^ delete toolbar 
 * 3. EBhEvV[W switch  lgwmSize, lgwmSysColorChange,
 *    lgwmNotify Ă͑֐ŌĂ
 */

/*
 * statusbar g߂ɂ
 * 1. RXgN^ statusbar = new lgcStatusbar() Ă
 * 2. fXgN^ delete statusbar 
 * 3. EBhEvV[W switch  lgwmSize, lgwmDrawItem Ă
 *    ֐ŌĂ
 */
 

#include "lgcToolbar.h"
#include "lgcStatusbar.h"


class lgMainWindow : public lgWindow
{
public:
  lgcStatusbar *statusbar;          // Xe[^Xo[
protected:
  lgcToolbar *toolbar;              // c[o[
  const char *help_filename;        // wvt@C
  DWORD (*context_help_id)[2];      // wv ID ƃRg[ ID ̑Ή\
  POINT toolbar_context_help_point; // toolbar ŃReLXgwvoʒu
  int default_return_value;         // Window Message ̃fBtHgԒl
  
public:
  lgMainWindow(HWND _hwnd)
    : lgWindow(_hwnd),
      statusbar(NULL),
      toolbar(NULL),
      help_filename(NULL),
      context_help_id(NULL),
      default_return_value(0)
  { }
  
  // Window Message
  
  // R}h
  int wmCommand(int notify_code, int id, HWND hwnd_control);
  
  // ReLXgwv
  int wmCommand_ContextHelp(void);

  // wv
  int wmCommand_Help(void);

  // ReLXgwvj[
  int wmContextMenu(HWND _hwnd, int x, int y);
  
  // I[i[h[
  int wmDrawItem(int id, DRAWITEMSTRUCT *dis);
  
  // wv
  int wmHelp(HELPINFO *hi);
  
  // j[Iꂽ
  int wmMenuSelect(UINT item, UINT flags, HMENU hmenu);
  
  // 点
  int wmNotify(int id, NMHDR *nmhdr);
  
  // ToolTip eLXg̗v
  int wmNotify_TtnNeedText(TOOLTIPTEXT *ttt);
  
  // ToolTip 
  int wmNotify_TtnPop(NMHDR *nmhdr);

  // EBhETCYω
  int wmSize(int size_type, int width, int height);
  
  // VXeJ[ω
  int wmSysColorChange(void);
  
  // ̑
  
  // context_help_id  help_filename ̐ݒ
  const char *SetHelp(const char *_help_filename,
		      DWORD (*_context_help_id)[2])
  {
    context_help_id = _context_help_id;
    const char *old = help_filename;
    help_filename = _help_filename;
    return old;
  }
};


// WndProc
#define lgwc ((lgWindowClass*)GetWindowLongPtr(hwnd, GWLP_USERDATA))

// EBhEł
// int wmCreate(CREATESTRUCT *cs)
#define lgwmCreate							\
case WM_CREATE:								\
SetWindowLongPtr(hwnd, GWLP_USERDATA, (LONG_PTR)new lgWindowClass(hwnd));	\
return lgwc->wmCreate((CREATESTRUCT *)lParam);

// EBhEȂȂ
#define lgwmDestroy				\
case WM_DESTROY:				\
delete lgwc;					\
return 0;

// R}h
// int wmCommand(int notify_code, int id, HWND hwnd_control)
#define lgwmCommand							\
case WM_COMMAND:							\
return lgwc->wmCommand(HIWORD(wParam), LOWORD(wParam), (HWND)lParam);

// 
// int wmChar(TCHAR char_code, long key_data)
#define lgwmChar				\
case WM_CHAR:					\
return lgwc->wmChar((TCHAR)wParam, lParam);

// t@Chbvꂽ
// int wmDropFiles(HANDLE hdrop)
#define lgwmDropFiles				\
case WM_DROPFILES:				\
return lgwc->wmDropFiles((HANDLE)wParam);

// j[Iꂽ
// int wmMenuSelect(UINT item, UINT flags, HMENU hmenu)
#define lgwmMenuSelect							\
case WM_MENUSELECT:							\
return lgwc->wmMenuSelect((UINT)LOWORD(wParam), (UINT)HIWORD(wParam),	\
			(HMENU)lParam);

// m点
// int wmNotify(int id, NMHDR *nmhdr)
#define lgwmNotify					\
case WM_NOTIFY:						\
return lgwc->wmNotify((int)wParam, (NMHDR *)lParam);

// ĕ``
// int wmPaint(void)
#define lgwmPaint				\
case WM_PAINT:					\
return lgwc->wmPaint();

// pbgω
// int wmPaletteChanged(HWND hwnd)
#define lgwmPaletteChanged			\
case WM_PALETTECHANGED:				\
return lgwc->wmPaletteChanged((HWND)wParam);

// pbgύX`X
// int wmQueryNewPalette(void)
#define lgwmQueryNewPalette			\
case WM_QUERYNEWPALETTE:			\
return lgwc->wmQueryNewPalette();

// tH[JX󂯎
// int wmSetFocus(HWND hwnd_lose_focus)
#define lgwmSetFocus				\
case WM_SETFOCUS:				\
return lgwc->wmSetFocus((HWND)wParam);

// EBhETCYω
// int wmSize(int size_type, int width, int height)
#define lgwmSize				\
case WM_SIZE:					\
return lgwc->wmSize((int)wParam, LOWORD(lParam), HIWORD(lParam));

// VXeJ[ω
// int wmSysColorChange(void)
#define lgwmSysColorChange			\
case WM_SYSCOLORCHANGE:				\
return lgwc->wmSysColorChange();

// VXeR}h
// int wmSysCommand(int command_type, int x, int y)
#define lgwmSysCommand							 \
case WM_SYSCOMMAND:							 \
return lgwc->wmSysCommand(wParam,					 \
			  (short)LOWORD(lParam), (short)HIWORD(lParam));

// ^C}[
// int wmTimer(int id, TIMERPROC *timer_proc)
#define lgwmTimer					\
case WM_TIMER:						\
return lgwc->wmTimer(wParam, (TIMERPROC *)lParam);

// {^
// int wm?Button???(WORD keys, int x, int y)
#define lgwm_Button_(message, func)					 \
case message:								 \
return lgwc->func(wParam, (short)LOWORD(lParam), (short)HIWORD(lParam));
#define lgwmLButtonUp lgwm_Button_(WM_LBUTTONUP, wmLButtonUp)
#define lgwmMButtonUp lgwm_Button_(WM_MBUTTONUP, wmMButtonUp)
#define lgwmRButtonUp lgwm_Button_(WM_RBUTTONUP, wmRButtonUp)
#define lgwmLButtonDown lgwm_Button_(WM_LBUTTONDOWN, wmLButtonDown)
#define lgwmMButtonDown lgwm_Button_(WM_MBUTTONDOWN, wmMButtonDown)
#define lgwmRButtonDown lgwm_Button_(WM_RBUTTONDOWN, wmRButtonDown)
#define lgwmLButtonDblclk lgwm_Button_(WM_LBUTTONDBLCLK, wmLButtonDblclk)
#define lgwmMButtonDblclk lgwm_Button_(WM_MBUTTONDBLCLK, wmMButtonDblclk)
#define lgwmRButtonDblclk lgwm_Button_(WM_RBUTTONDBLCLK, wmRButtonDblclk)

// }EXړ
// int wmMouseMove(WORD keys, int x, int y)
#define lgwmMouseMove				\
case WM_MOUSEMOVE:				\
return lgwc->wmMouseMove(wParam, GET_X_LPARAM(lParam), GET_Y_LPARAM(lParam));

#define lgwmNCMouseMove				\
case WM_NCMOUSEMOVE:			\
return lgwc->wmNcMouseMove(wParam, GET_X_LPARAM(lParam), GET_Y_LPARAM(lParam));

#define	lgwmSetCursor				\
case WM_SETCURSOR:				\
return lgwc->wmSetCursor(wParam, LOWORD(lParam), HIWORD(lParam));

// 
// int wmClose(void)
#define lgwmClose				\
case WM_CLOSE:					\
return lgwc->wmClose();

// EBhEeLXgݒ
// int wmSetText(char *text)
#define lgwmSetText				\
case WM_SETTEXT:				\
return lgwc->wmSetText((char *)lParam);

// _CAO{bNX̏
// BOOL wmInitDialog(HWND focus, LPARAM lParam)
#define lgwmInitDialog							\
case WM_INITDIALOG:							\
SetWindowLongPtr(hwnd, GWLP_USERDATA, (LONG_PTR)new lgWindowClass(hwnd));	\
return lgwc->wmInitDialog((HWND)wParam, lParam);

// wv̕\
// int wmHelp(HELPINFO *hi)
#define lgwmHelp				\
case WM_HELP:					\
return lgwc->wmHelp((HELPINFO *)lParam);

// ReLXgj[
// int wmCotextMenu(HWND hwnd, int x, int y)
#define lgwmContextMenu							  \
case WM_CONTEXTMENU:							  \
return lgwc->wmContextMenu((HWND)wParam,				  \
			   (short)LOWORD(lParam), (short)HIWORD(lParam));

// I[i[h[
// int wmDrawItem(int id, DRAWITEMSTRUCT *dis)
#define lgwmDrawItem						\
case WM_DRAWITEM:						\
return lgwc->wmDrawItem((int)wParam, (DRAWITEMSTRUCT *)lParam);

// \
// int wmShowWindow(BOOL does_become_showon, int status)
#define lgwmShowWindow					\
case WM_SHOWWINDOW:					\
return lgwc->wmShowWindow((BOOL)wParam, (int)lParam);

// XN[o[
// int wmHScroll(int code, int pos, HWND hwnd_scrollbar)
// int wmVScroll(int code, int pos, HWND hwnd_scrollbar)
#define lgwmHScroll							\
case WM_HSCROLL:							\
return lgwc->wmHScroll((short)LOWORD(wParam), (short)HIWORD(wParam),	\
		       (HWND)lParam);
#define lgwmVScroll							\
case WM_VSCROLL:							\
return lgwc->wmVScroll((short)LOWORD(wParam), (short)HIWORD(wParam),	\
		       (HWND)lParam);

// Rg[̐F
// HBRUSH wmCtlColorBtn(HDC hdc, HWND hwnd)
#define lgwmCtlColorBtn						\
case WM_CTLCOLORBTN:						\
return (int)lgwc->wmCtlColorBtn((HDC)(wParam), (HWND)(lParam));
// HBRUSH wmCtlColorDlg(HDC hdc, HWND hwnd)
#define lgwmCtlColorDlg						\
case WM_CTLCOLORDLG:						\
return (int)lgwc->wmCtlColorDlg((HDC)(wParam), (HWND)(lParam));
// HBRUSH wmCtlColorEdit(HDC hdc, HWND hwnd)
#define lgwmCtlColorEdit						\
case WM_CTLCOLOREDIT:							\
return (int)lgwc->wmCtlColorEdit((HDC)(wParam), (HWND)(lParam));
// HBRUSH wmCtlColorListBox(HDC hdc, HWND hwnd)
#define lgwmCtlColorListBox						\
case WM_CTLCOLORLISTBOX:						\
return (int)lgwc->wmCtlColorListBox((HDC)(wParam), (HWND)(lParam));
// HBRUSH wmCtlColorMsgBox(HDC hdc, HWND hwnd)
#define lgwmCtlColorMsgBox						\
case WM_CTLCOLORMSGBOX:							\
return (int)lgwc->wmCtlColorMsgBox((HDC)(wParam), (HWND)(lParam));
// HBRUSH wmCtlColorScrollBar(HDC hdc, HWND hwnd)
#define lgwmCtlColorScrollBar						\
case WM_CTLCOLORSCROLLBAR:						\
return (int)lgwc->wmCtlColorScrollBar((HDC)(wParam), (HWND)(lParam));
// HBRUSH wmCtlColorStatic(HDC hdc, HWND hwnd)
#define lgwmCtlColorStatic						\
case WM_CTLCOLORSTATIC:							\
return (int)lgwc->wmCtlColorStatic((HDC)(wParam), (HWND)(lParam));

// L[R[h
// int wmKeyUp(int vk_code, long key_data)
#define lgwmKeyUp				\
case WM_KEYUP:					\
return lgwc->wmKeyUp((int)wParam, lParam)
// int wmSysKeyUp(int vk_code, long key_data)
#define lgwmSysKeyUp				\
case WM_SYSKEYUP:				\
return lgwc->wmSysKeyUp((int)wParam, lParam)
// int wmKeyDown(int vk_code, long key_data)
#define lgwmKeyDown				\
case WM_KEYDOWN:				\
return lgwc->wmKeyDown((int)wParam, lParam)
// int wmSysKeyDown(int vk_code, long key_data)
#define lgwmSysKeyDown				\
case WM_SYSKEYDOWN:				\
return lgwc->wmSysKeyDown((int)wParam, lParam)
